package view;

import java.awt.Toolkit;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.io.IOException;
import java.io.PipedInputStream;
import java.io.PipedOutputStream;
import java.net.URL;
import javax.swing.*;
import model.Direction;
import networking.Display;
import networking.GameBoardClient;
import networking.GameBoardServer;

public class GameWindow extends JFrame implements KeyListener, Display {
    final PipedInputStream[] client_in = new PipedInputStream[2];
    final PipedInputStream[] server_in = new PipedInputStream[2];
    final PipedOutputStream[] client_out = new PipedOutputStream[2];
    final PipedOutputStream[] server_out = new PipedOutputStream[2];
    final GameBoardClient[] clients = new GameBoardClient[2];
    final GameBoardServer server;

    /**
     * Creates new form GameWindow
     */
    public GameWindow() throws IOException {
        for (int i = 0; i < 2; i++){
            // Server side in  <-- Client side out
            // Server side out --> Client side in
            client_in[i] = new PipedInputStream();
            client_out[i] = new PipedOutputStream();
            server_in[i] = new PipedInputStream(client_out[i]);
            server_out[i] = new PipedOutputStream(client_in[i]);
        }
        server = new GameBoardServer(25, server_in, server_out);

        initComponents();
        URL url = GameWindow.class.getClassLoader().getResource("res/pacman.png");
        setIconImage(Toolkit.getDefaultToolkit().getImage(url));
        setLocationRelativeTo(null);
        addKeyListener(this);
        
        addWindowListener(new WindowAdapter() {
            @Override
            public void windowClosing(WindowEvent we) {
                server.stop();
                clients[0].stop();
                clients[1].stop();
                System.exit(0);
            }
        });

        // Clients has to start AFTER the server is started, otherwise their
        // constructor will block on their InputStream
        server.start();
        clients[0] = new GameBoardClient(client_in[0], client_out[0], this); // display
        clients[1] = new GameBoardClient(client_in[1], client_out[1], null); // no display
        for (int i = 0; i < 2; i++){
            clients[i].start();
        }
    }

    /**
     * This method is called from within the constructor to initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is always
     * regenerated by the Form Editor.
     */
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        gamePanel1 = new GamePanel(server);

        setDefaultCloseOperation(javax.swing.WindowConstants.DO_NOTHING_ON_CLOSE);
        setTitle("PacMan");
        setResizable(false);

        gamePanel1.setBackground(new java.awt.Color(255, 255, 255));

        javax.swing.GroupLayout gamePanel1Layout = new javax.swing.GroupLayout(gamePanel1);
        gamePanel1.setLayout(gamePanel1Layout);
        gamePanel1Layout.setHorizontalGroup(
            gamePanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGap(0, 833, Short.MAX_VALUE)
        );
        gamePanel1Layout.setVerticalGroup(
            gamePanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGap(0, 354, Short.MAX_VALUE)
        );

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(getContentPane());
        getContentPane().setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addContainerGap()
                .addComponent(gamePanel1, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                .addContainerGap())
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addContainerGap()
                .addComponent(gamePanel1, javax.swing.GroupLayout.DEFAULT_SIZE, 354, Short.MAX_VALUE)
                .addContainerGap())
        );

        pack();
    }// </editor-fold>//GEN-END:initComponents

    /**
     * @param args the command line arguments
     */
    public static void main(String args[]) {
        /* Set the Nimbus look and feel */
        //<editor-fold defaultstate="collapsed" desc=" Look and feel setting code (optional) ">
        /* If Nimbus (introduced in Java SE 6) is not available, stay with the default look and feel.
         * For details see http://download.oracle.com/javase/tutorial/uiswing/lookandfeel/plaf.html 
         */
        try {
            for (javax.swing.UIManager.LookAndFeelInfo info : javax.swing.UIManager.getInstalledLookAndFeels()) {
                if ("Windows".equals(info.getName())) {
                    javax.swing.UIManager.setLookAndFeel(info.getClassName());
                    break;
                }
            }
        } catch (ClassNotFoundException ex) {
            java.util.logging.Logger.getLogger(GameWindow.class.getName()).log(java.util.logging.Level.SEVERE, null, ex);
        } catch (InstantiationException ex) {
            java.util.logging.Logger.getLogger(GameWindow.class.getName()).log(java.util.logging.Level.SEVERE, null, ex);
        } catch (IllegalAccessException ex) {
            java.util.logging.Logger.getLogger(GameWindow.class.getName()).log(java.util.logging.Level.SEVERE, null, ex);
        } catch (javax.swing.UnsupportedLookAndFeelException ex) {
            java.util.logging.Logger.getLogger(GameWindow.class.getName()).log(java.util.logging.Level.SEVERE, null, ex);
        }
        //</editor-fold>

        /* Create and display the form */
        java.awt.EventQueue.invokeLater(new Runnable() {
            public void run() {
                try {
                    new GameWindow().setVisible(true);
                } catch (IOException ex) {}
            }
        });
    }

    // Variables declaration - do not modify//GEN-BEGIN:variables
    private view.GamePanel gamePanel1;
    // End of variables declaration//GEN-END:variables

    @Override
    public void keyTyped(KeyEvent ke) {
    }

    @Override
    public void keyPressed(KeyEvent ke) {
        String str = "";
        try {
            if (ke.getKeyCode() == KeyEvent.VK_A){
                clients[0].changePlayerDirection(Direction.LEFT);
            } else if (ke.getKeyCode() == KeyEvent.VK_D){
                clients[0].changePlayerDirection(Direction.RIGHT);
            } else if (ke.getKeyCode() == KeyEvent.VK_W){
                clients[0].changePlayerDirection(Direction.UP);
            } else if (ke.getKeyCode() == KeyEvent.VK_S){
                clients[0].changePlayerDirection(Direction.DOWN);
            } else if (ke.getKeyCode() == KeyEvent.VK_LEFT){
                clients[1].changePlayerDirection(Direction.LEFT);
            } else if (ke.getKeyCode() == KeyEvent.VK_RIGHT){
                clients[1].changePlayerDirection(Direction.RIGHT);
            } else if (ke.getKeyCode() == KeyEvent.VK_UP){
                clients[1].changePlayerDirection(Direction.UP);
            } else if (ke.getKeyCode() == KeyEvent.VK_DOWN){
                clients[1].changePlayerDirection(Direction.DOWN);
            } else if (ke.getKeyCode() == KeyEvent.VK_0){
                server_in[0].close();
                server_out[0].close();
                client_in[0].close();
                client_out[0].close();
            } else if (ke.getKeyCode() == KeyEvent.VK_1){
                server_in[1].close();
                server_out[1].close();
                client_in[1].close();
                client_out[1].close();
            }
        } catch (IOException e){}
    }

    @Override
    public void keyReleased(KeyEvent ke) {
    }

    @Override
    public void refresh() {
        gamePanel1.repaint();
    }

    @Override
    public void connectionLost(int client) {
        JOptionPane.showMessageDialog(this, "Connection lost to client-" + client, "Connection error", JOptionPane.ERROR_MESSAGE);
    }

    @Override
    public void connectionLost() {
        JOptionPane.showMessageDialog(this, "Connection lost to server", "Connection error", JOptionPane.ERROR_MESSAGE);
    }
}
